unit f_main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Grids, rStringGridEd, PropSaveMain, PropSaveGrids,
  rImprovedComps, rHTMLLabel;

type
  TMain = class(TForm)
    PSFile: TPropSaveFile;
    PSMain: TPropSaveMain;
    GridsPropSave1: TGridsPropSave;
    mFList: TrStringsHolder;
    mFile: TrStringsHolder;
    gbClient: TrGroupBox;
    Label1: TLabel;
    eDir: TEdit;
    btSelDir: TButton;
    btRun: TButton;
    gr: TrStringGridEd;
    m: TMemo;
    Label2: TLabel;
    chBackup: TCheckBox;
    btLoad: TrURLLabel;
    btSave: TrURLLabel;
    OpenDlg: TOpenDialog;
    SaveDlg: TSaveDialog;
    btClear: TrURLLabel;
    procedure btSelDirClick(Sender: TObject);
    procedure btRunClick(Sender: TObject);
    procedure btClearClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btLoadClick(Sender: TObject);
    procedure btSaveClick(Sender: TObject);
  private
    BkDir: string;
    procedure ProcDFM(fn: string);
    function ProcObj(cn, pn, old, new: string; del: boolean): integer;
  public
  end;

var
  Main: TMain;

implementation

{$R *.dfm}

uses StrUtils, rtool, rdlg, rfiletool, rDialogsEx;

procedure TMain.FormCreate(Sender: TObject);
begin
  OpenDlg.InitialDir:=GetAppDir;
  SaveDlg.InitialDir:=GetAppDir;
end;

procedure TMain.btSelDirClick(Sender: TObject);
var
  s: string;
begin
  s:=rDialogsEx.BrowseForFolderEx('', self.eDir.Text);
  if s <> '' then
    self.eDir.Text:=s;
end;

procedure TMain.btRunClick(Sender: TObject);
var
  s: string;
  I: integer;
begin
  m.Lines.clear;
  // prepare dirs
  s:=Trim(self.eDir.Text);
  s:=rtool.SetEndOfPath(s);
  BkDir:=SetEndOfPath(s + '_backup_log\' + FormatDateTime('YYYYMMDD_hhmmss', Now));
  ForceDirectories(BkDir);
  // load DFM file list
  rfiletool.LoadFileListSubDir(mFList.Strings, s, '*.dfm');
  // proc all files
  for I:=0 to mFList.Strings.Count - 1 do
  begin
    ProcDFM(mFList.Strings[I]);
  end;
  m.Lines.Add('FINISHED.');
  m.Lines.SaveToFile(BkDir + '_log.txt');
end;

procedure TMain.ProcDFM(fn: string);
var
  r, upd: integer;
begin
  // load DFM file
  m.Lines.Add(fn);
  mFile.Strings.LoadFromFile(fn);
  // proc all rows of setting grid
  upd:=0;
  for r:=1 to gr.RowCount - 2 do
  begin
    upd:=upd + ProcObj(gr.Cells[0, r], gr.Cells[1, r], gr.Cells[2, r], gr.Cells[3, r], gr.BoolCells[4, r]);
  end;
  // save changes if updated
  if upd > 0 then
  begin
    // create backup
    if chBackup.Checked then
      CopyFile(PChar(fn), PChar(BkDir + ChangeFileExt(ExtractFileName(fn), '.df_bak')), true);
    // remove empty lines
    for r:=mFile.Strings.Count - 1 downto 0 do
      if mFile.Strings[r] = '' then
        mFile.Strings.Delete(r);
    // save updated DFM file
    mFile.Strings.SaveToFile(fn);
  end;
end;

function TMain.ProcObj(cn: string; pn: string; old: string; new: string; del: boolean): integer;
var
  s, obj, prop, val: string;
  isObj, proc: boolean;
  r: integer;
begin
  Result:=0;
  isObj:=false;
  // proc all DFM lines
  for r:=0 to mFile.Strings.Count - 1 do
  begin
    s:=Trim(mFile.Strings[r]);

    if not isObj and (StrUtils.StartsText('object', s) or StrUtils.StartsText('inherited', s)) then
    begin
      // ObjectName : ClassName [x]
      obj:=Trim(rtool.RemoveStrPart(s, ':', true));
      s:=Trim(s);
      // remove [x] if exists
      s:=Trim(rtool.FindStrPart(s,' ',1));
      if SameText(s, cn) then
      begin
        // correct classname = start of object def
        isObj:=true;
        // m.Lines.Add(Format('Object %s found on line %d', [cn, r + 1]))
      end;
    end;
    if isObj and SameText('end', s) then
    begin
      // end of object
      isObj:=false;
      // m.Lines.Add(Format('Object %s end on line %d', [cn, r + 1]))
    end;
    if isObj then
    begin
      // PropName = Value
      prop:=Trim(rtool.RemoveStrPart(s, '=', true));
      val:=Trim(s);
      if SameText(prop, pn) then
      begin
        // correct property, log
        m.Lines.Add(Format('Line %d, Object: %s, Property %s, Value: %s', [r + 1, obj, prop, val]));
        // compare current value
        proc:=(old = '') or SameText(old, val);
        if proc then
        begin
          if del then
          begin
            mFile.Strings[r]:='';
            m.Lines.Add('DELETED.');
            Inc(Result);
          end
          else
            if (new <> '') then
            begin
              mFile.Strings[r]:=Format('%s = %s', [prop, new]);
              m.Lines.Add(Format('SET NEW VALUE: %s.', [new]));
              Inc(Result);
            end;
        end;
      end;
    end;
  end;
end;

procedure TMain.btClearClick(Sender: TObject);
begin
  gr.ClearEntireTable();
end;

procedure TMain.btLoadClick(Sender: TObject);
begin
  if OpenDlg.Execute then
  begin
    PSFile.LoadFile(OpenDlg.FileName);
    PSMain.ReadProperties;
  end;
end;

procedure TMain.btSaveClick(Sender: TObject);
begin
  if SaveDlg.Execute then
  begin
    PSMain.SaveProperties;
    PSFile.SaveFile(SaveDlg.FileName);
  end;
end;

end.
